/* GENERAL STYLES */
.imageit-container{
    display:flex;
    flex-direction: column;
    width:100%;
}

.imageit-pseudo-selector{
    display:none;
    width: calc(100% - 3rem - 2px) !important;
    padding: .5rem 1.5rem !important;
    background-color: rgb(121, 174, 200);
    border: 1px solid rgb(73, 164, 209);
    border-radius: 4px;
    color:white;
    text-align:center;
    font-weight:600;
    cursor:pointer;
}

.imageit-selector-icon{
    height: 1.5em;
    margin-right: 1.5em;
}

.imageit-preview{
    max-width: 100%;
    margin: 1.5rem 0px;
    padding: 0px .5rem;
    min-height: 75px;
}

.imageit-preview-content{
    display:flex;
    width: 100%;
    justify-content: space-between;
    align-items: center;
}

.imageit-clear-button{
    display: flex;
    align-items: center;
    justify-content: center;
    width: 2rem;
    height: 2rem;
    color: #ff3466;
    border: 2px solid #ff3466;
    border-radius: 50%;
    cursor: pointer;
}

.imageit-undo-button{
    display: flex;
    align-items: center;
    justify-content: center;
    padding: .5rem 1.5rem !important;
    background-color: rgb(121, 174, 200);
    border: 1px solid rgb(73, 164, 209);
    border-radius: 4px;
    cursor: pointer;
    color: white;
}

.imageit-text-light{
    color: #a4b7bd;
}

.imageit-inactive{
    display: none;
}

.imageit-error{
    width: 100%;
    padding: 1rem 0px;
    text-align: center;
    color: #ff3466;
}


/* SCALEIT STYLES */
.imageit-preview-text{
    flex-grow: 1;
    flex-shrink: 100;
    padding: 0px 1rem;
}

.imageit-preview-text p{
    word-break: break-all;
}

.imageit-preview-image{
    max-width:150px;
    max-height:150px;
    box-shadow: 2px 2px 5px rgba(0,0,0,0.4);
}



/* CROPPER STYLES */
.imageit-preview-cropit-content{
    flex-direction: column;
    justify-content: space-between;
    align-items: center;
}

.imageit-cropit-container{
    width: 100%;
}

.imageit-cropper-content{
    display: flex;
    flex-direction: column;
    max-width: 100%;
    min-height: 75px;
}

.imageit-cropper-image-container{
    max-height:80vh;
}

.imageit-cropper-image{
    max-width:100%;
    max-height:100%;
}

.imageit-clear-button.imageit-cancel-crop{
    width: calc(100% - 4px);
    height: 2rem;
    margin-bottom: 1.5rem;
    color: #ff3466;
    border: 2px solid #ff3466;
    border-radius: 4px;
    text-align: center;
    cursor: pointer;
}





/* For loading */
.loading{
    position:relative;
    opacity:0.5;
 }
 
 .loading::before{
    content: '';
    position:absolute;
    display:flex;
    top:-5%;
    left:-5%;
    width:110%;
    height:110%;
    border-radius:5px;
 }

.loading::after{
    position:absolute;
    top:calc(50% - 35px);
    left:calc(50% - 35px);
    width: 70px;
    height: 70px;
    border-style: solid;
    border-top-color: yellow;
    border-right-color: yellow;
    border-left-color: transparent;
    border-bottom-color: transparent;
    border-radius: 50%;
    box-sizing: border-box;
    animation: rotate 3s ease-in-out infinite;
    transform: rotate(-200deg);
}

@keyframes rotate {
0% { border-width: 10px; border-top-color: yellow; border-right-color: yellow;}
25% { border-width: 3px; }
50% { 
    transform: rotate(115deg); 
    border-width: 10px;
    border-top-color: pink; border-right-color: pink;
}
75% { border-width: 3px;}
100% { border-width: 10px; border-top-color: yellow; border-right-color: yellow;}
}