<div class="imageit-cropit-container">
    {% spaceless %}
        {% for widget in widget.subwidgets %}
            {% include widget.template_name %}
        {% endfor %}
    {% endspaceless %}
</div>
